def wrap_file_with_fastapi(
    input_file_path, output_file_path, endpoint_path, http_method="post"
):
    # Read the input file
    with open(input_file_path, "r") as file:
        original_code = file.read()

    # Extract the main logic into a new function
    agent_function_logic = """
    def run_agent_task():
        # Original agent logic
        agent = Agent(
            llm=llm,
            max_loops=1,
            autosave=True,
            dashboard=True,
        )
        return agent.run("Generate a 10,000 word blog on health and wellness.")
        """

    # Full template with FastAPI and agent_api_wrapper
    fastapi_template = f"""import os
    from dotenv import load_dotenv
    from fastapi import FastAPI
    from swarms.models import OpenAIChat
    from swarms.structs import Agent
    from your_module_where_decorator_is import agent_api_wrapper

    # Load the environment variables
    load_dotenv()

    # Get the API key from the environment
    api_key = os.environ.get("OPENAI_API_KEY")

    # Initialize the language model
    llm = OpenAIChat(
        temperature=0.5,
        model_name="gpt-4",
        openai_api_key=api_key,
    )

    app = FastAPI()

    @agent_api_wrapper(Agent, app, path="{endpoint_path}", http_method="{http_method}")
    {agent_function_logic}

    # Original code without the agent initialization and run logic
    {original_code}
    """

    # Remove the original Agent initialization and run logic from the original code
    fastapi_template = fastapi_template.replace("## Initialize the workflow\n", "")
    fastapi_template = fastapi_template.replace(
        "agent = Agent(\n    llm=llm,\n    max_loops=1,\n    autosave=True,\n    dashboard=True,\n)\n\n",
        "",
    )
    fastapi_template = fastapi_template.replace(
        '# Run the workflow on a task\nout = agent.run("Generate a 10,000 word blog on health and wellness.")\nprint(out)',
        "",
    )

    # Write the modified code to a new output file
    with open(output_file_path, "w") as file:
        file.write(fastapi_template)


# Example usage
wrap_file_with_fastapi("input_file.py", "output_file.py", "/run-agent")



from swarms_cloud.main import agent_api_wrapper
from swarms_cloud.rate_limiter import rate_limiter
from swarms_cloud.func_api_wrapper import api_wrapper, FuncAPIWrapper

__all__ = [
    "agent_api_wrapper",
    "rate_limiter",
    "api_wrapper",
    "FuncAPIWrapper",
]



import logging
from typing import Callable, Type, Optional, Any

from fastapi import FastAPI, HTTPException, Request
from swarms.structs.agent import Agent

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)


def agent_api_wrapper(
    agent_class: Type[Agent],
    app: FastAPI,
    path: Optional[str] = None,
    http_method: Optional[str] = "get",
    logging: bool = False,
    response_formatter: Callable[[Any], Any] = None,
    *args,
    **kwargs,
):
    """Expose agent methods as API endpoints

    Args:
        agent_class (Type[Agent]): Agent class
        app (FastAPI): FastAPI instance
        path (Optional[str], optional): Path to register the endpoint at. Defaults to None.
        http_method (Optional[str], optional): HTTP method to use. Defaults to "get".
        logging (bool, optional): Whether to log the request. Defaults to False.
        response_formatter (Callable[[Any], Any], optional): Formatter for the response. Defaults to None.

    Example:
    >>> from swarms.agents import Agent
    >>> from fastapi import FastAPI
    >>> app = FastAPI()
    >>> @agent_api_wrapper(Agent, app, "/agent")
    ... def agent_method():
    ...     return "Hello World"

    """

    def decorator(func: Callable):
        async def endpoint_wrapper(*args, **kwargs):
            try:
                # Create an instance of the agent class
                logger.info(
                    f"Creating instance of {agent_class.__name__} with args: {args} and kwargs: {kwargs}"
                )
                agent_instance = agent_class(*args, **kwargs)

                # Call the method of the agent class
                logger.info(f"Calling method {func.__name__} of {agent_class.__name__}")
                result = getattr(agent_instance, func.__name__)()
                logger.info(
                    f"Method {func.__name__} of {agent_class.__name__} returned: {result}"
                )

                # Format the response if a formatter is provided
                if response_formatter:
                    result = response_formatter(result)

                return result
            except Exception as error:
                logger.error(f"An error occurred: {str(error)}")
                raise HTTPException(status_code=500, detail=str(error))

        if http_method.lower() == "get":
            logger.info(f"Registering GET endpoint at {path}")
            app.get(path)(endpoint_wrapper)
        elif http_method.lower() == "post":
            logger.info(f"Registering POST endpoint at {path}")
            app.post(path)(endpoint_wrapper)
        return func

    return decorator



import secrets
import string


def generate_api_key(prefix: str = "sk-", length: int = 50, *args, **kwargs):
    """Generate a random api key

    Args:
        prefix (str, optional): _description_. Defaults to "sk-".
        length (int, optional): _description_. Defaults to 50.

    Raises:
        ValueError: _description_
        RuntimeError: _description_

    Returns:
        _type_: _description_


    Example:
    >>> from swarms_cloud.api_key_generator import generate_api_key
    >>> generate_api_key()
    >>> sk-9a7b8c5d6e7f8g9h0i1j2k3l4m5n6o7p8q9r0s1t2u3v4w5x6y7z8A9B0C1D2E3F4G5H6I7J8K9L0M1N2O3P4Q5R6S7T8U9V0W1X2Y3Z4
    """
    if length <= len(prefix):
        raise ValueError("Length must be greater than prefix length")

    try:
        # Generate a random string of letters for the api
        characters = string.ascii_letters + string.digits
        secure_key = "".join(secrets.choice(characters) for _ in range(length))
        return prefix + secure_key
    except Exception as error:
        # Handle unexpeccted errors
        raise RuntimeError(f"Error generating api key: {error}")



import ast
import astor


class CodeTransformer(ast.NodeTransformer):
    def __init__(self):
        self.agent_logic = []

    def visit_Assign(self, node):
        if (
            isinstance(node.value, ast.Call)
            and getattr(node.value.func, "id", "") == "Agent"
        ):
            self.agent_logic.append(node)
            return None
        return node

    def visit_Expr(self, node):
        if (
            isinstance(node.value, ast.Call)
            and getattr(node.value.func, "id", "") == "Agent"
        ):
            self.agent_logic.append(node)
            return None
        return node


def wrap_with_fastapi(input_code):
    tree = ast.parse(input_code)
    transformer = CodeTransformer()
    transformer.visit(tree)

    # Define the agent_method function
    agent_function = ast.FunctionDef(
        name="agent_method",
        args=ast.arguments(
            args=[],
            vararg=None,
            kwarg=None,
            posonlyargs=[],
            kwonlyargs=[],
            kw_defaults=[],
            defaults=[],
        ),
        body=transformer.agent_logic,
        decorator_list=[
            ast.Call(
                func=ast.Attribute(
                    value=ast.Name(id="app", ctx=ast.Load()),
                    attr="post",
                    ctx=ast.Load(),
                ),
                args=[],
                keywords=[ast.keyword(arg="path", value=ast.Str(s="/agent"))],
            )
        ],
    )

    # Add necessary imports
    imports = [
        ast.Import(names=[ast.alias(name="FastAPI", asname=None)]),
        ast.ImportFrom(
            module="swarms.structs.agent",
            names=[ast.alias(name="Agent", asname=None)],
            level=0,
        ),
        ast.ImportFrom(
            module="swarms_cloud",
            names=[ast.alias(name="agent_api_wrapper", asname=None)],
            level=0,
        ),
        ast.ImportFrom(
            module="uvicorn", names=[ast.alias(name="uvicorn", asname=None)], level=0
        ),
    ]

    # Insert imports at the beginning
    for imp in reversed(imports):
        tree.body.insert(0, imp)

    # Insert FastAPI app initialization
    tree.body.insert(
        len(imports),
        ast.Assign(
            targets=[ast.Name(id="app", ctx=ast.Store())],
            value=ast.Call(
                func=ast.Name(id="FastAPI", ctx=ast.Load()), args=[], keywords=[]
            ),
        ),
    )

    # Add the agent_method function
    tree.body.append(agent_function)

    # Code to run the app with uvicorn
    run_app_code = ast.parse(
        "if __name__ == '__main__':\n    uvicorn.run(app, host='0.0.0.0', port=8000)"
    ).body
    tree.body.extend(run_app_code)

    return astor.to_source(tree)


# Example Usage
input_code = """



import os

from dotenv import load_dotenv

# Import the OpenAIChat model and the Agent struct
from swarms.models import OpenAIChat
from swarms.structs import Agent

# Load the environment variables
load_dotenv()

# Get the API key from the environment
api_key = os.environ.get("OPENAI_API_KEY")

# Initialize the language model
llm = OpenAIChat(
    temperature=0.5,
    model_name="gpt-4",
    openai_api_key=api_key,
)


## Initialize the workflow
agent = Agent(
    llm=llm,
    max_loops=1,
    autosave=True,
    dashboard=True,
)

# Run the workflow on a task
out = agent.run("Generate a 10,000 word blog on health and wellness.")
print(out)



"""
output_code = wrap_with_fastapi(input_code)
print(output_code)



import logging
from typing import Callable

import uvicorn
from fastapi import FastAPI, HTTPException

# Logger initialization
logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)


# Function API Wrapper for functions: [FUNCTION]
def api_wrapper(
    app: FastAPI,
    path: str,
    http_method: str = "post",
):
    """API Wrapper

    Args:
        app (FastAPI): _description_
        path (str): _description_
        http_method (str, optional): _description_. Defaults to "post".

    Example:
    >>> app = FastAPI()
    >>> @api_wrapper(app, "/endpoint")
    ... def endpoint():
    ...     return "Hello World"
    >>> uvicorn.run(app, host="")
    """

    def decorator(func: Callable):
        try:

            async def endpoint_wrapper(*args, **kwargs):
                try:
                    # Call the func with provided args
                    logger.info(f"Calling method {func.__name__}")
                    result = func(*args, **kwargs)
                    return result
                except Exception as error:
                    logger.error(f"Error in {func.__name__}: {error}")
                    raise HTTPException(status_code=500, detail=str(error))

            # Register the endpoint
            endpoint_func = getattr(app, http_method)
            endpoint_func(path)(endpoint_wrapper)
            return func
        except Exception as error:
            print(f"Error in {func.__name__}: {error}")

    return decorator


# Function API Wrapper for functions: [CLASS]
class FuncAPIWrapper:
    """Functional API Wrapper


    Args:
        host (str, optional): Host to run the API on. Defaults to "
        port (int, optional): Port to run the API on. Defaults to 8000.

    Methods:
        add: Add an endpoint to the API
        run: Run the API

    Example:
    >>> api = FuncAPIWrapper()
    >>> @api.add("/endpoint")
    ... def endpoint():
    ...     return "Hello World"
    >>> api.run()


    """

    def __init__(self, host: str = "0.0.0.0", port: int = 8000, *args, **kwargs):
        self.host = host
        self.port = port
        self.app = FastAPI()

    def add(self, path: str, method: str = "post", *args, **kwargs):
        """Add an endpoint to the API

        Args:
            path (str): _description_
            method (str, optional): _description_. Defaults to "post".
        """

        def decorator(func: Callable):
            try:
                if method.lower() == "get":
                    endpoint_func = self.app.get(path)
                elif method.lower() == "post":
                    endpoint_func = self.app.post(path)
                else:
                    raise ValueError(f"Invalid method: {method}")

                @endpoint_func
                async def endpoint_wrapper(*args, **kwargs):
                    try:
                        logger.info(f"Calling method {func.__name__}")
                        result = func(*args, **kwargs)
                        return result
                    except Exception as error:
                        logger.error(f"Error in {func.__name__}: {error}")
                        raise HTTPException(status_code=500, detail=str(error))

                # Register the endpoint with the http method
                endpoint_func = getattr(self.app, method.lower())
                endpoint_func(path)(endpoint_wrapper)
                return func
            except Exception as error:
                logger.info(f"Error in {func.__name__}: {error}")

        return decorator

    def run(self, *args, **kwargs):
        """Run the API

        Args:

        """
        uvicorn.run(self.app, host=self.host, port=self.port)

    def __call__(self, *args, **kwargs):
        """Call the run method

        Args:

        """
        self.run(*args, **kwargs)
